/*
    c99tree - parse tree output layer
    Copyright (C) 2010-2015  Tibor Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "c99tree-cc.h"
#include "tree_print.h"
#include "pathgrow.h"
#include "escaping.h"

int C99Nprint_ind(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	char *tmp;
	int t;
	char locbuff[1024];

	switch(state) {
		case PS_OPENING:
			tmp = c99c_escape(node->name);
			if (node->loc.twin_parent != NULL)
				t = node->loc.twin_parent->uid;
			else
				t = -1;
			printf("%s%s %s [%d:%d] (%s)\n", prefix, c99c_node_type_names[node->type], tmp, node->uid, t, c99c_loc_print(ctx, node, locbuff, sizeof(locbuff)));
			free(tmp);
			break;
		case PS_HEAD:
			printf("FILE  (%s)\n", ctx->loc_format);
			break;
		case PS_TAIL:
		case PS_CLOSING:
			break;
	}
	return 0;
}

int C99Nprint_xml(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	char *loc_tmp;
	char locbuff[1024];

	switch(state) {
		case PS_OPENING:
			printf("%s<%s", prefix, c99c_node_type_names[node->type]);
			if (node->name != NULL) {
				char *tmp;
				tmp = c99c_escape_xml(node->name);
				printf(" name=\"%s\"", tmp);
				free(tmp);
			}

			loc_tmp = c99c_escape_xml(c99c_loc_print(ctx, node, locbuff, sizeof(locbuff)));
			printf(" location=\"%s\"", loc_tmp);
			free(loc_tmp);

			printf(" uid=\"%d\"", node->uid);
			if (node->loc.twin_parent != NULL)
				printf(" twin_parent=\"%d\"", node->loc.twin_parent->uid);

			if ((node->children == NULL) && (!ctx->format_long))
				printf("/");
			printf(">\n");
		break;
		case PS_CLOSING:
			if ((node->children != NULL) || (ctx->format_long))
				printf("%s</%s>\n", prefix, c99c_node_type_names[node->type]);
			break;
		case PS_HEAD:
			printf("<?xml version=\"1.0\" ?>\n");
			printf("<FILE location_format=\"%s\">\n", c99c_escape_xml(ctx->loc_format));
			break;
		case PS_TAIL:
			printf("</FILE>\n");
			break;
	}
	return 0;
}

int C99Nprint_plist(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	char locbuff[1024];

	switch(state) {
		case PS_OPENING:
			printf("%s%s = {\n", prefix, c99c_node_type_names[node->type]);
			if (node->name != NULL) {
				char *tmp;
				tmp = c99c_escape(node->name);
				printf("%s name = \"%s\";\n", prefix, tmp);
				free(tmp);
			}
			printf("%s location = \"%s\";\n", prefix, c99c_loc_print(ctx, node, locbuff, sizeof(locbuff)));

			printf("%s uid = \"%d\";\n", prefix, node->uid);
			if (node->loc.twin_parent != NULL)
				printf("%s twin_parent = \"%d\";\n", prefix, node->loc.twin_parent->uid);

			break;
		case PS_CLOSING:
			printf("%s};\n", prefix);
			break;
		case PS_HEAD:
			printf("FILE = {\n");
			printf(" globals = {\n");
			printf("  location_format = \"%s\";\n", c99c_escape(ctx->loc_format));
			printf(" };\n");
			break;
		case PS_TAIL:
			printf("};\n");
			break;

	}
	return 0;
}

int C99Nprint_json(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	char locbuff[1024];

	switch(state) {
		case PS_OPENING:
			printf("%s{ \"type\": \"%s\", ", prefix, c99c_node_type_names[node->type]);
			if (node->name != NULL) {
				char *tmp;
				tmp = c99c_escape(node->name);
				printf("\"name\": \"%s\", ", tmp);
				free(tmp);
			}
			printf("\"uid\": \"%d\", ", node->uid);
			if (node->loc.twin_parent != NULL)
				printf("\"twin_parent\": \"%d\", ", node->loc.twin_parent->uid);

			printf("\"location\": \"%s\"%s ", c99c_loc_print(ctx, node, locbuff, sizeof(locbuff)), (node->children ? "," : ""));
		
			if (node->children)
				printf("\"children\": [\n");
			else
				printf("}%s\n", (node->next ? "," : ""));
			break;

		case PS_CLOSING:
			if (node->children)
				printf("%s]}%s\n", prefix, (node->next ? "," : ""));
			break;

		case PS_HEAD:
			printf("%s[\n", prefix);
			break;

		case PS_TAIL:
			printf("%s]\n", prefix);
			break;
	}
	return 0;
}

int C99Nprint_perl(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	char locbuff[1024];

	switch(state) {
		case PS_OPENING:
			printf("%s{ \"type\" => \"%s\", ", prefix, c99c_node_type_names[node->type]);
			if (node->name != NULL) {
				char *tmp;
				tmp = c99c_escape(node->name);
				printf("\"name\" => \"%s\", ", tmp);
				free(tmp);
			}

			printf("\"uid\" => \"%d\", ", node->uid);
			if (node->loc.twin_parent != NULL)
				printf("\"twin_parent\" => \"%d\", ", node->loc.twin_parent->uid);

			printf("\"location\" => \"%s\"%s ", c99c_loc_print(ctx, node, locbuff, sizeof(locbuff)), (node->children ? "," : ""));
			if (node->children)
				printf("\"children\" => [\n");
			else
				printf("}%s\n", (node->next ? "," : ""));
			break;

		case PS_CLOSING:
			if (node->children)
				printf("%s]}%s\n", prefix, (node->next ? "," : ""));
			break;

		case PS_HEAD:
			printf("%s[\n", prefix);
			break;

		case PS_TAIL:
			printf("%s]\n", prefix);
			break;
	}
	return 0;
}

int C99Nprint_web3(c99tree_cc_t *ctx, void **nprint_ctx, const char *prefix, const c99c_node_t *node, print_state_t state)
{
	static c99c_pathgrow_t *tpath, *ipath;
	static int uid;
	char uids[64];
	char locbuff[1024];

	switch(state) {
		case PS_HEAD:
			uid = 0;
			tpath = c99c_pathgrow_alloc('/');
			ipath = c99c_pathgrow_alloc('/');
			printf("|ID=0|TAG=globals|TPATH=/globals|IPATH=/globals|location_format=%s\n", ctx->loc_format);
			break;
		case PS_OPENING:
			uid++;
			sprintf(uids, "%d", uid);
			c99c_pathgrow_append(tpath, c99c_node_type_names[node->type]);
			c99c_pathgrow_append(ipath, uids);
			printf("|ID=%d|TAG=%s|TPATH=%s|IPATH=%s|", uid, c99c_node_type_names[node->type], tpath->path, ipath->path);
			if (node->name != NULL) {
				char *tmp;
				tmp = c99c_escape(node->name);
				printf("name=%s|", tmp);
				free(tmp);
			}

			printf("uid=%d|", node->uid);
			if (node->loc.twin_parent != NULL)
				printf("twin_parent=%d|", node->loc.twin_parent->uid);

			printf("loc=%s|loc_file=%s|loc_line=%d|\n", c99c_loc_print(ctx, node, locbuff, sizeof(locbuff)), node->loc.file->fn, node->loc.sline);
			break;
		case PS_CLOSING:
			c99c_pathgrow_truncate(tpath);
			c99c_pathgrow_truncate(ipath);
			break;
		case PS_TAIL:
			c99c_pathgrow_free(tpath);
			c99c_pathgrow_free(ipath);
			break;
	}
	return 0;
}

